@@ -5,9 +5,10 @@ use warnings;
 package MooseX::Deprecated;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.004';
+our $VERSION   = '0.005';
 
 use Carp;
+use Devel::Callsite 0.08;
 use MooseX::Role::Parameterized;
 
 our @CARP_NOT = qw(
@@ -41,8 +42,13 @@ my $deprecated = sub
 	# Skip over any Moose internals
 	local %Carp::Internal = %Carp::Internal;
 	my $i = 0;
-	/\A(Moose|Class..MOP|Test::Warnings|MooseX::Deprecated|Eval::Closure)\b/ && ++$Carp::Internal{$_}
-		while defined($_ = caller $i++);
+	my $site;
+	/\A(Moose|Class..MOP|Test::Warnings|MooseX::Deprecated|Eval::Closure)\b/
+		? ++$Carp::Internal{$_}
+		: ($site ||= callsite($i-1))
+		while defined($_ = caller($i++));
+	
+	return if $already{$site}++;
 	
 	warnings::warnif deprecated => EDEPRECATED($name, $type);
 };
@@ -72,7 +78,9 @@ role {
 			
 			before $method_name => sub
 			{
-				unshift @_, $method_type => $method_name;
+				unshift @_ => (
+					$method_type => $method_name,
+				);
 				goto $deprecated;
 			}
 		}
@@ -81,15 +89,24 @@ role {
 	method BUILD => sub { };
 	after BUILD => sub
 	{
-		exists($_[1]{$_}) && $deprecated->(argument => $_, @_)
-			for sort keys %init_args;
+		my $immutable;
+		for (sort keys %init_args)
+		{
+			$immutable = !!(ref($_[0])->meta->is_immutable) unless defined($immutable);
+			$deprecated->(
+				argument => $_,
+				@_,
+			) if exists($_[1]{$_});
+		}
 	};
 	
 	for my $method (@methods)
 	{
 		before $method => sub
 		{
-			unshift @_, method => $method;
+			unshift @_ => (
+				method => $method,
+			);
 			goto $deprecated;
 		}
 	}
@@ -154,6 +171,8 @@ Warnings can be upgraded to fatal errors with:
 
    use warnings FATAL => qw( deprecated );
 
+Warnings will only be issued once per call site.
+
 When consuming the role you I<must> pass either a list of attributes,
 or a list of methods, or both, as parameters to the role. If you forget
 to do so, you'll get an error message like:
@@ -219,7 +238,8 @@ considered to be the caller.
 
 In the test suite I just skip the complex test that checks for this on
 Perl prior to 5.10, allowing you to install this module without a hitch
-on Perl 5.8.
+on Perl 5.8. B<< However, you are strongly discouraged from using this
+module with Perl 5.8. >>
 
 =head1 SEE ALSO
 
@@ -245,7 +265,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
